@@ -992,7 +992,7 @@ auto backtrack16(s16info_s * s,
 
       uint64_t const d = *((uint64_t *) (dirbuffer +
                                    (offset + (16 * s->qlen * (j / 4)) +
-                                    (16 * i) + (4 * (j & 3)) % dirbuffersize)));
+                                    (16 * i) + (4 * (j & 3))) % dirbuffersize));
 
       if ((s->op == 'I') && (d & maskextleft))
         {
