@@ -30,22 +30,23 @@
 #include "config.h"
 #include <city.h>
 
-#include <algorithm>
-#include <cstring>  // for memcpy and memset
-#include <utility> // make_pair
+#include <algorithm>  // std::swap
+#include <cstdint>  // int32_t
+#include <cstring>  // std::memcpy, std::memset
+#include <utility> // std::pair, std::make_pair
 
 
 using namespace std;  // refactoring: bad practice
 
-static uint64 UNALIGNED_LOAD64(const char *p) {
-  uint64 result;
-  memcpy(&result, p, sizeof(result));
+static auto UNALIGNED_LOAD64(const char *p) -> uint64 {
+  uint64 result = 0;
+  std::memcpy(&result, p, sizeof(result));
   return result;
 }
 
-static uint32 UNALIGNED_LOAD32(const char *p) {
-  uint32 result;
-  memcpy(&result, p, sizeof(result));
+static auto UNALIGNED_LOAD32(const char *p) -> uint32 {
+  uint32 result = 0;
+  std::memcpy(&result, p, sizeof(result));
   return result;
 }
 
@@ -99,11 +100,11 @@ static uint32 UNALIGNED_LOAD32(const char *p) {
 #endif
 #endif
 
-static uint64 Fetch64(const char *p) {
+static auto Fetch64(const char *p) -> uint64 {
   return uint64_in_expected_order(UNALIGNED_LOAD64(p));
 }
 
-static uint32 Fetch32(const char *p) {
+static auto Fetch32(const char *p) -> uint32 {
   return uint32_in_expected_order(UNALIGNED_LOAD32(p));
 }
 
@@ -117,17 +118,17 @@ static const uint32_t c1 = 0xcc9e2d51;
 static const uint32_t c2 = 0x1b873593;
 
 // A 32-bit to 32-bit integer hash copied from Murmur3.
-static uint32 fmix(uint32 h)
+static auto fmix(uint32 h) -> uint32
 {
-  h ^= h >> 16;
+  h ^= h >> 16U;
   h *= 0x85ebca6b;
-  h ^= h >> 13;
+  h ^= h >> 13U;
   h *= 0xc2b2ae35;
-  h ^= h >> 16;
+  h ^= h >> 16U;
   return h;
 }
 
-static uint32 Rotate32(uint32 val, int shift) {
+static auto Rotate32(uint32 val, int shift) -> uint32 {
   // Avoid shifting by 32: doing so yields an undefined result.
   return shift == 0 ? val : ((val >> shift) | (val << (32 - shift)));
 }
@@ -135,21 +136,21 @@ static uint32 Rotate32(uint32 val, int shift) {
 #undef PERMUTE3
 #define PERMUTE3(a, b, c) do { std::swap(a, b); std::swap(a, c); } while (0)
 
-static uint32 Mur(uint32 a, uint32 h) {
+static auto Mur(uint32 a, uint32 h) -> uint32 {
   // Helper from Murmur3 for combining two 32-bit values.
   a *= c1;
   a = Rotate32(a, 17);
   a *= c2;
   h ^= a;
   h = Rotate32(h, 19);
-  return h * 5 + 0xe6546b64;
+  return (h * 5) + 0xe6546b64;
 }
 
-static uint32 Hash32Len13to24(const char *s, size_t len) {
-  const uint32 a = Fetch32(s - 4 + (len >> 1));
+static auto Hash32Len13to24(const char * s, size_t len) -> uint32 {
+  const uint32 a = Fetch32(s - 4 + (len >> 1U));
   const uint32 b = Fetch32(s + 4);
   const uint32 c = Fetch32(s + len - 8);
-  const uint32 d = Fetch32(s + (len >> 1));
+  const uint32 d = Fetch32(s + (len >> 1U));
   const uint32 e = Fetch32(s);
   const uint32 f = Fetch32(s + len - 4);
   const uint32 h = len;
@@ -157,7 +158,7 @@ static uint32 Hash32Len13to24(const char *s, size_t len) {
   return fmix(Mur(f, Mur(e, Mur(d, Mur(c, Mur(b, Mur(a, h)))))));
 }
 
-static uint32 Hash32Len0to4(const char *s, size_t len) {
+static auto Hash32Len0to4(const char * s, size_t len) -> uint32 {
   uint32 b = 0;
   uint32 c = 9;
   for (int i = 0; i < len; i++) {
@@ -168,18 +169,18 @@ static uint32 Hash32Len0to4(const char *s, size_t len) {
   return fmix(Mur(b, Mur(len, c)));
 }
 
-static uint32 Hash32Len5to12(const char *s, size_t len) {
+static auto Hash32Len5to12(const char * s, size_t len) -> uint32 {
   uint32 a = len;
   uint32 b = len * 5;
   uint32 c = 9;
-  uint32 d = b;
+  uint32 const d = b;
   a += Fetch32(s);
   b += Fetch32(s + len - 4);
-  c += Fetch32(s + ((len >> 1) & 4));
+  c += Fetch32(s + ((len >> 1U) & 4U));
   return fmix(Mur(c, Mur(b, Mur(a, d))));
 }
 
-uint32 CityHash32(const char *s, size_t len) {
+auto CityHash32(const char * s, size_t len) -> uint32 {
   if (len <= 24) {
     return len <= 12 ?
         (len <= 4 ? Hash32Len0to4(s, len) : Hash32Len5to12(s, len)) :
@@ -252,58 +253,58 @@ uint32 CityHash32(const char *s, size_t len) {
 
 // Bitwise right rotate.  Normally this will compile to a single
 // instruction, especially if the shift is a manifest constant.
-static uint64 Rotate(uint64 val, int shift) {
+static auto Rotate(uint64 val, int shift) -> uint64 {
   // Avoid shifting by 64: doing so yields an undefined result.
-  return shift == 0 ? val : ((val >> shift) | (val << (64 - shift)));
+  return shift == 0 ? val : ((val >> shift) | (val << (64U - shift)));
 }
 
-static uint64 ShiftMix(uint64 val) {
-  return val ^ (val >> 47);
+static auto ShiftMix(uint64 val) -> uint64 {
+  return val ^ (val >> 47U);
 }
 
-static uint64 HashLen16(uint64 u, uint64 v) {
+static auto HashLen16(uint64 u, uint64 v) -> uint64 {
   return Hash128to64(uint128(u, v));
 }
 
-static uint64 HashLen16(uint64 u, uint64 v, uint64 mul) {
+static auto HashLen16(uint64 u, uint64 v, uint64 mul) -> uint64 {
   // Murmur-inspired hashing.
   uint64 a = (u ^ v) * mul;
-  a ^= (a >> 47);
+  a ^= (a >> 47U);
   uint64 b = (v ^ a) * mul;
-  b ^= (b >> 47);
+  b ^= (b >> 47U);
   b *= mul;
   return b;
 }
 
-static uint64 HashLen0to16(const char *s, size_t len) {
+static auto HashLen0to16(const char *s, size_t len) -> uint64 {
   if (len >= 8) {
-    const uint64 mul = k2 + len * 2;
+    const uint64 mul = k2 + (len * 2);
     const uint64 a = Fetch64(s) + k2;
     const uint64 b = Fetch64(s + len - 8);
-    const uint64 c = Rotate(b, 37) * mul + a;
+    const uint64 c = (Rotate(b, 37) * mul) + a;
     const uint64 d = (Rotate(a, 25) + b) * mul;
     return HashLen16(c, d, mul);
   }
   if (len >= 4) {
-    const uint64 mul = k2 + len * 2;
+    const uint64 mul = k2 + (len * 2);
     const uint64 a = Fetch32(s);
-    return HashLen16(len + (a << 3), Fetch32(s + len - 4), mul);
+    return HashLen16(len + (a << 3U), Fetch32(s + len - 4), mul);
   }
   if (len > 0) {
     const uint8 a = s[0];
-    const uint8 b = s[len >> 1];
+    const uint8 b = s[len >> 1U];
     const uint8 c = s[len - 1];
-    const uint32 y = static_cast<uint32>(a) + (static_cast<uint32>(b) << 8);
-    const uint32 z = len + (static_cast<uint32>(c) << 2);
-    return ShiftMix(y * k2 ^ z * k0) * k2;
+    const uint32 y = static_cast<uint32>(a) + (static_cast<uint32>(b) << 8U);
+    const uint32 z = len + (static_cast<uint32>(c) << 2U);
+    return ShiftMix((y * k2) ^ (z * k0)) * k2;
   }
   return k2;
 }
 
 // This probably works well for 16-byte strings as well, but it may be overkill
 // in that case.
-static uint64 HashLen17to32(const char *s, size_t len) {
-  const uint64 mul = k2 + len * 2;
+static auto HashLen17to32(const char *s, size_t len) -> uint64 {
+  const uint64 mul = k2 + (len * 2);
   const uint64 a = Fetch64(s) * k1;
   const uint64 b = Fetch64(s + 8);
   const uint64 c = Fetch64(s + len - 8) * mul;
@@ -314,8 +315,9 @@ static uint64 HashLen17to32(const char *s, size_t len) {
 
 // Return a 16-byte hash for 48 bytes.  Quick and dirty.
 // Callers do best to use "random-looking" values for a and b.
-static pair<uint64, uint64> WeakHashLen32WithSeeds(
-    uint64 w, uint64 x, uint64 y, uint64 z, uint64 a, uint64 b) {
+static auto WeakHashLen32WithSeeds(uint64 w, uint64 x, uint64 y, uint64 z,
+                                   uint64 a, uint64 b)
+  -> std::pair<uint64, uint64> {
   a += w;
   b = Rotate(b + a + z, 21);
   const uint64 c = a;
@@ -326,8 +328,8 @@ static pair<uint64, uint64> WeakHashLen32WithSeeds(
 }
 
 // Return a 16-byte hash for s[0] ... s[31], a, and b.  Quick and dirty.
-static pair<uint64, uint64> WeakHashLen32WithSeeds(
-    const char* s, uint64 a, uint64 b) {
+static auto WeakHashLen32WithSeeds(const char *s, uint64 a, uint64 b)
+  -> std::pair<uint64, uint64> {
   return WeakHashLen32WithSeeds(Fetch64(s),
                                 Fetch64(s + 8),
                                 Fetch64(s + 16),
@@ -337,8 +339,8 @@ static pair<uint64, uint64> WeakHashLen32WithSeeds(
 }
 
 // Return an 8-byte hash for 33 to 64 bytes.
-static uint64 HashLen33to64(const char *s, size_t len) {
-  const uint64 mul = k2 + len * 2;
+static auto HashLen33to64(const char *s, size_t len) -> uint64 {
+  const uint64 mul = k2 + (len * 2);
   uint64 a = Fetch64(s) * k2;
   uint64 b = Fetch64(s + 8);
   const uint64 c = Fetch64(s + len - 24);
@@ -347,18 +349,18 @@ static uint64 HashLen33to64(const char *s, size_t len) {
   const uint64 f = Fetch64(s + 24) * 9;
   const uint64 g = Fetch64(s + len - 8);
   const uint64 h = Fetch64(s + len - 16) * mul;
-  const uint64 u = Rotate(a + g, 43) + (Rotate(b, 30) + c) * 9;
+  const uint64 u = Rotate(a + g, 43) + ((Rotate(b, 30) + c) * 9);
   const uint64 v = ((a + g) ^ d) + f + 1;
   const uint64 w = bswap_64((u + v) * mul) + h;
   const uint64 x = Rotate(e + f, 42) + c;
   const uint64 y = (bswap_64((v + w) * mul) + g) * mul;
   const uint64 z = e + f + c;
-  a = bswap_64((x + z) * mul + y) + b;
-  b = ShiftMix((z + a) * mul + d + h) * mul;
+  a = bswap_64(((x + z) * mul) + y) + b;
+  b = ShiftMix(((z + a) * mul) + d + h) * mul;
   return b + x;
 }
 
-uint64 CityHash64(const char *s, size_t len) {
+auto CityHash64(const char *s, size_t len) -> uint64 {
   if (len <= 32) {
     if (len <= 16) {
       return HashLen0to16(s, len);
@@ -374,8 +376,8 @@ uint64 CityHash64(const char *s, size_t len) {
   uint64 x = Fetch64(s + len - 40);
   uint64 y = Fetch64(s + len - 16) + Fetch64(s + len - 56);
   uint64 z = HashLen16(Fetch64(s + len - 48) + len, Fetch64(s + len - 24));
-  pair<uint64, uint64> v = WeakHashLen32WithSeeds(s + len - 64, len, z);
-  pair<uint64, uint64> w = WeakHashLen32WithSeeds(s + len - 32, y + k1, x);
+  auto v = WeakHashLen32WithSeeds(s + len - 64, len, z);
+  auto w = WeakHashLen32WithSeeds(s + len - 32, y + k1, x);
   x = x * k1 + Fetch64(s);
 
   // Decrease len to the nearest multiple of 64, and operate on 64-byte chunks.
@@ -392,22 +394,22 @@ uint64 CityHash64(const char *s, size_t len) {
     s += 64;
     len -= 64;
   } while (len != 0);
-  return HashLen16(HashLen16(v.first, w.first) + ShiftMix(y) * k1 + z,
+  return HashLen16(HashLen16(v.first, w.first) + (ShiftMix(y) * k1) + z,
                    HashLen16(v.second, w.second) + x);
 }
 
-uint64 CityHash64WithSeed(const char *s, size_t len, uint64 seed) {
+auto CityHash64WithSeed(const char *s, size_t len, uint64 seed) -> uint64 {
   return CityHash64WithSeeds(s, len, k2, seed);
 }
 
-uint64 CityHash64WithSeeds(const char *s, size_t len,
-                           uint64 seed0, uint64 seed1) {
+auto CityHash64WithSeeds(const char *s, size_t len,
+                         uint64 seed0, uint64 seed1) -> uint64 {
   return HashLen16(CityHash64(s, len) - seed0, seed1);
 }
 
 // A subroutine for CityHash128().  Returns a decent 128-bit hash for strings
 // of any length representable in signed long.  Based on City and Murmur.
-static uint128 CityMurmur(const char *s, size_t len, uint128 seed) {
+static auto CityMurmur(const char *s, size_t len, uint128 seed) -> uint128 {
   uint64 a = Uint128Low64(seed);
   uint64 b = Uint128High64(seed);
   uint64 c = 0;
@@ -437,7 +439,7 @@ static uint128 CityMurmur(const char *s, size_t len, uint128 seed) {
   return uint128(a ^ b, HashLen16(b, a));
 }
 
-uint128 CityHash128WithSeed(const char *s, size_t len, uint128 seed) {
+auto CityHash128WithSeed(const char *s, size_t len, uint128 seed) -> uint128 {
   if (len < 128) {
     return CityMurmur(s, len, seed);
   }
@@ -501,7 +503,7 @@ uint128 CityHash128WithSeed(const char *s, size_t len, uint128 seed) {
                  HashLen16(x + w.second, y + v.second));
 }
 
-uint128 CityHash128(const char *s, size_t len) {
+auto CityHash128(const char *s, size_t len) -> uint128 {
   return len >= 16 ?
       CityHash128WithSeed(s + 16, len - 16,
                           uint128(Fetch64(s), Fetch64(s + 8) + k0)) :
@@ -513,8 +515,8 @@ uint128 CityHash128(const char *s, size_t len) {
 #include <nmmintrin.h>
 
 // Requires len >= 240.
-static void CityHashCrc256Long(const char *s, size_t len,
-                               uint32 seed, uint64 *result) {
+static auto CityHashCrc256Long(const char *s, size_t len,
+                               uint32 seed, uint64 *result) -> void {
   uint64 a = Fetch64(s + 56) + k0;
   uint64 b = Fetch64(s + 96) + k0;
   uint64 c = result[0] = HashLen16(b, len);
@@ -601,14 +603,14 @@ static void CityHashCrc256Long(const char *s, size_t len,
 }
 
 // Requires len < 240.
-static void CityHashCrc256Short(const char *s, size_t len, uint64 *result) {
+static auto CityHashCrc256Short(const char *s, size_t len, uint64 *result) -> void {
   char buf[240];
   memcpy(buf, s, len);
   memset(buf + len, 0, 240 - len);
   CityHashCrc256Long(buf, 240, ~static_cast<uint32>(len), result);
 }
 
-void CityHashCrc256(const char *s, size_t len, uint64 *result) {
+auto CityHashCrc256(const char *s, size_t len, uint64 *result) -> void {
   if (LIKELY(len >= 240)) {
     CityHashCrc256Long(s, len, 0, result);
   } else {
@@ -616,7 +618,7 @@ void CityHashCrc256(const char *s, size_t len, uint64 *result) {
   }
 }
 
-uint128 CityHashCrc128WithSeed(const char *s, size_t len, uint128 seed) {
+auto CityHashCrc128WithSeed(const char *s, size_t len, uint128 seed) -> uint128 {
   if (len <= 900) {
     return CityHash128WithSeed(s, len, seed);
   } else {
@@ -629,7 +631,7 @@ uint128 CityHashCrc128WithSeed(const char *s, size_t len, uint128 seed) {
   }
 }
 
-uint128 CityHashCrc128(const char *s, size_t len) {
+auto CityHashCrc128(const char *s, size_t len) -> uint128 {
   if (len <= 900) {
     return CityHash128(s, len);
   } else {
