@@ -58,15 +58,18 @@
 
 */
 
+using count_t = unsigned short;
+
+
 #ifdef __x86_64__
-void increment_counters_from_bitmap_sse2(count_t * counters,
+auto increment_counters_from_bitmap_sse2(count_t * counters,
                                          unsigned char * bitmap,
-                                         unsigned int totalbits);
-void increment_counters_from_bitmap_ssse3(count_t * counters,
+                                         unsigned int totalbits) -> void;
+auto increment_counters_from_bitmap_ssse3(count_t * counters,
                                           unsigned char * bitmap,
-                                          unsigned int totalbits);
+                                          unsigned int totalbits) -> void;
 #else
-void increment_counters_from_bitmap(count_t * counters,
+auto increment_counters_from_bitmap(count_t * counters,
                                     unsigned char * bitmap,
-                                    unsigned int totalbits);
+                                    unsigned int totalbits) -> void;
 #endif
