@@ -58,69 +58,73 @@
 
 */
 
+#include <cstdint>  // uint64_t
+#include <cstdio>  // std::size_t
+
+
 struct seqinfo_s
 {
-  size_t header_p;
-  size_t seq_p;
-  size_t qual_p;
+  std::size_t header_p;
+  std::size_t seq_p;
+  std::size_t qual_p;
   unsigned int headerlen;
   unsigned int seqlen;
   unsigned int size;
 };
 
-typedef struct seqinfo_s seqinfo_t;
+using seqinfo_t = struct seqinfo_s;
 
 extern char * datap;
 extern seqinfo_t * seqindex;
 
-inline char * db_getheader(uint64_t seqno)
+inline auto db_getheader(uint64_t seqno) -> char *
 {
   return datap + seqindex[seqno].header_p;
 }
 
-inline char * db_getsequence(uint64_t seqno)
+inline auto db_getsequence(uint64_t seqno) -> char *
 {
   return datap + seqindex[seqno].seq_p;
 }
 
-inline uint64_t db_getabundance(uint64_t seqno)
+inline auto db_getabundance(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].size;
 }
 
-inline uint64_t db_getsequencelen(uint64_t seqno)
+inline auto db_getsequencelen(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].seqlen;
 }
 
-inline uint64_t db_getheaderlen(uint64_t seqno)
+inline auto db_getheaderlen(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].headerlen;
 }
 
-void db_read(const char * filename, int upcase);
-void db_free();
+auto db_read(const char * filename, int upcase) -> void;
+auto db_free() -> void;
 
-uint64_t db_getsequencecount();
-uint64_t db_getnucleotidecount();
-uint64_t db_getlongestheader();
-uint64_t db_getlongestsequence();
-uint64_t db_getshortestsequence();
+auto db_getsequencecount() -> uint64_t;
+auto db_getnucleotidecount() -> uint64_t;
+auto db_getlongestheader() -> uint64_t;
+auto db_getlongestsequence() -> uint64_t;
+auto db_getshortestsequence() -> uint64_t;
 
 /* Note: the sorting functions below must be called after db_read,
    but before dbindex_prepare */
 
-void db_sortbylength();
-void db_sortbylength_shortest_first();
+auto db_sortbylength() -> void;
+auto db_sortbylength_shortest_first() -> void;
 
-void db_sortbyabundance();
+auto db_sortbyabundance() -> void;
 
-bool db_is_fastq();
-char * db_getquality(uint64_t seqno);
+auto db_is_fastq() -> bool;
+auto db_getquality(uint64_t seqno) -> char *;
 
-void db_setinfo(bool new_is_fastq,
+auto db_setinfo(bool new_is_fastq,
                 uint64_t new_sequences,
                 uint64_t new_nucleotides,
                 uint64_t new_longest,
                 uint64_t new_shortest,
-                uint64_t new_longestheader);
+                uint64_t new_longestheader) -> void;
